#!/usr/bin/env python
#-*- coding:utf-8 -*-
#
#	Copyright (C) Humberto Fraga 2010 <xisberto@gmail.com>
#	
#	This file is part of MagicLibrary.
#	MagicLibrary is another deck manager for Collectible Card Game Magic: The Gathering.
#
#	MagicLibrary is free software: you can redistribute it and/or modify
#	it under the terms of the GNU Affero General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	MagicLibrary is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU Affero General Public License for more details.
#
#	You should have received a copy of the GNU Affero General Public License
#	along with MagicLibrary.  If not, see <http://www.gnu.org/licenses/>.

import urllib
from sgmllib import SGMLParser

class Gatherer:
	def __init__(self, controller):
		"""This class loads gatherer.wizards.com and parses data from there.
		"""
		self.controller = controller
		self.parser = GathererParser()

	def htmlRetrieve(self, url, report, complete_callback):
		self.parser.reset()
		(filename, headers) = urllib.urlretrieve(url, reporthook=report)
		sock = open(filename)
		html_source = sock.read()
		sock.close()
		self.parser.feed(html_source)
		self.parser.close()
		print "página principal, %s cards" % len(self.parser.cards)
		if self.parser.paging_list: #if there is a paging list, we should read other pages
			for pagenum in self.parser.paging_list[1:]: #according to Gatherer's sematics
				(filename, headers) = urllib.urlretrieve(url+"&page=%s" % (pagenum-1), reporthook=report)
				sock = open(filename)
				html_source = sock.read()
				sock.close()
				page_parser = GathererParser(ignore_pages=True)
				page_parser.feed(html_source)
				page_parser.close()
				self.parser.cards.extend(page_parser.cards)
		complete_callback()
	
	def imgRetrieve(self, url, filename, report, complete_callback):
		urllib.urlretrieve(url, filename, report)
		complete_callback(filename)

	def get(self, what):
		"""Accepts "format", "set" or "type". It's a sortcut to getFormats(), getSets() and getTypes()
		"""
		if what == "format":
			return self.getFormats()
		elif what == "set":
			return self.getSets()
		elif what == "type":
			return self.getTypes()
		else:
			return None

	def getFormats(self):
		return self.parser.formats
	def getSets(self):
		return self.parser.sets
	def getTypes(self):
		return self.parser.types
	def getCards(self):
		return self.parser.cards
	
class GathererParser(SGMLParser):
	"""Parses the HTML and extract data from it.
	"""
	def __init__(self, ignore_pages=False):
		"""	ignore_pages is used internally to parse each page of a big search individually
		"""
		SGMLParser.__init__(self)
		self.ignore_pages=ignore_pages

	def reset(self):
		"""Reset the parser, empty the list of options and the capture_options flag.
		"""
		SGMLParser.reset(self)
		self.a_found = False
		self.table_list_found = False
		self.tr_list_found = False
		self.td_list_found = ""
		self.table_details_found = False
		self.tr_details_found = False
		self.td_details_found = False
		self.td_image_found = False
		self.card_image_url = ""
		self.div_detail_found = ""
		self.div_detail_found_id = ""
		self.sets = []
		self.formats = []
		self.types = []
		self.cards = []
		self.capture_options = ""
		self.div_paging_found = False
		self.build_paging_list = False
		self.paging_list = []

	def start_select(self, attrs):
		"""Search for the select tags from search form, using their "name" attribute. Set self.capture_options to:
		"format", "set" or "type" when one of those selects are found
		"" when other selects are found.
		"""
		name = [v for k, v in attrs if k=="name"]
		#Possible values for name are:
		#ctl00$ctl00$MainContent$Content$SearchControls$formatAddText
		#ctl00$ctl00$MainContent$Content$SearchControls$setAddText
		#ctl00$ctl00$MainContent$Content$SearchControls$typeAddText
		match = name[0].split('$')[-1].split('A')[0]
		if match == "format" or match == "set" or match == "type":
			self.capture_options = match
		else:
			self.capture_options = ""
	def end_select(self):
		self.capture_options = ""

	def start_option(self, attrs):
		"""Search for option tags inside select tags from search form, using their "value" attribute. Builds self.formats, self.sets and self.types according to self.capture_options.
		"""
		if self.capture_options != "":
			value = [v for k, v in attrs if k=="value"][0]
			if self.capture_options == "format":
				self.formats.append(value)
			if self.capture_options == "set":
				self.sets.append(value)
			if self.capture_options == "type":
				self.types.append(value)
	
	def start_table(self, attrs):
		"""Search for a table with class=="compact". Set self.table_list_found to True when found.
		Search for a table with class=="cardDetails". Set self.table_details_found to True when found.
		"""
		try:
			table_class = [v for k, v in attrs if k=="class"][0]
		except:
			table_class = ""
		if table_class == "compact":
			self.table_list_found = True
		elif table_class == "cardDetails":
			self.table_details_found = True
	def end_table(self):
		self.table_list_found = False
		self.table_details_found = False
	
	def start_tr(self, attrs):
		"""If self.table_list_found, search for a tr with class="cardItem". Set self.tr_list_found to True when found.
		The only tr when self.table_details_found has no class.
		"""
		if self.table_list_found:
			tr_class = [v for k, v in attrs if k=="class"][0]
			if tr_class.split(' ')[0] == "cardItem":
				self.tr_list_found = True
		elif self.table_details_found:
			self.tr_details_found = True
	def end_tr(self):
		self.tr_list_found = False
		self.tr_details_found = False
	
	def start_td(self, attrs):
		"""If self.tr_list_found, search for a td with class "name top", "mana top", "type top" and set self.td_list_found according.
		If self.tr_details_found, search tor a td with class "leftCol" or "rightCol".
		"""
		if self.tr_list_found:
			td_class = [v for k, v in attrs if k=="class"][0]
			if td_class == "name top":
				self.td_list_found = "name"
			elif td_class == "mana top":
				self.td_list_found = "mana"
			elif td_class == "type top":
				self.td_list_found = "type"
		elif self.tr_details_found:
			td_class = [v for k,v in attrs if k=="class"][0]
			if td_class == "leftCol":
				self.td_image_found = True
				self.td_details_found = False
			elif td_class == "rightCol":
				self.td_image_found = False
				self.td_details_found = True
	def end_td(self):
		self.td_list_found = ""

	def start_a(self, attrs):
		"""Card names are text elements in a tags inside td class="name top" tags. This sets self.a_found to True when those a tag is found.
		Paging links forces us to get through each page.
		"""
		if self.td_list_found:
			if [v for k, v in attrs if k=="id"][0].split('_')[-1]=="cardTitle":
				self.a_found = True
				self.card_multiverseid = [v for k, v in attrs if k=="href"][0].split('=')[-1]
		elif self.div_paging_found and not self.ignore_pages:
			self.build_paging_list = True
	def end_a(self):
		self.a_found = False
		self.build_paging_list = False
	
	def start_div(self, attrs):
		try:
			div_class = [v for k, v in attrs if k=="class"][0]
		except IndexError:
			div_class = ""
		if self.td_details_found:
			if div_class == "row":
				try:
					div_id = [v for k,v in attrs if k=="id"][0]
					div_id = div_id.split("_")[-1].split("R")[0]
				except IndexError:
					div_id = ""
				if div_id == "name" or div_id == "type" or div_id == "mana":
					self.div_detail_found_id = div_id
				else:
					self.div_detail_found_id = ""
			elif div_class == "value":
				self.div_detail_found = self.div_detail_found_id
		else:
			if div_class == "":
				try:
					div_id = [v for k, v in attrs if k=="id"][0]
				except IndexError:
					div_id = ""
				if div_id == "ctl00_ctl00_ctl00_MainContent_SubContent_bottomPagingControlsContainer":
					self.div_paging_found = True
	def end_div(self):
		self.div_detail_found = ""

	def start_img(self, attrs):
		"""Card costs are img elements inside td class="mana top". In their src attribute is the mana symbol.
		If parser is in a man td, append part of src attribute as a item to "mana" list inside the last dict in self.cards list (the last row is created/appended when the card name is found in self.handle_data)
		"""
		if self.td_list_found == "mana" or self.div_detail_found == "mana":
			symbol_name = [v for k, v in attrs if k=="src"][0]
			symbol_name = symbol_name.split("&")[-2].split("=")[-1]
			self.cards[-1]["mana"].append(symbol_name)
		elif self.td_image_found:
			self.card_image_url = "http://gatherer.wizards.com%s" % [v for k, v in attrs if k=="src"][0].split('..')[-1]
			self.card_multiverseid = self.card_image_url.split('&')[0].split('=')[-1]
			self.td_image_found = False

	def handle_data(self, text):
		"""Card names are text elements in a tags; card types are text elements in td tags.
		If parser is in a name td and a tag is found, appends a new dict to self.cards list.
		If parser is in a type td, inserts "type" item to last dict in self.cards list.
		"""
		if (self.td_list_found == "name" and self.a_found) or self.div_detail_found == "name":
			self.cards.append({"name":text.strip(), "mana":[], "multiverseid":self.card_multiverseid})
		elif self.td_list_found == "type" or self.div_detail_found == "type":
			self.cards[-1]["type"] = text.strip("\r\n ").replace(" \xe2\x80\x94",'-')
		elif self.build_paging_list and text.isdigit():
			self.paging_list.append(int(text))

if __name__=="__main__":
	gt = GathererParser(True)
	print gt.ignore_pages
